@@ -25,6 +25,8 @@
 #include "dzl-debug.h"
 
 #include "animation/dzl-animation.h"
+#include "shortcuts/dzl-shortcut-controller.h"
+#include "shortcuts/dzl-shortcut-private.h"
 #include "suggestions/dzl-suggestion.h"
 #include "suggestions/dzl-suggestion-entry.h"
 #include "suggestions/dzl-suggestion-popover.h"
@@ -35,6 +37,8 @@
 #include "widgets/dzl-list-box.h"
 #include "widgets/dzl-list-box-private.h"
 
+#define DELAYED_POPDOWN_MSEC 100
+
 struct _DzlSuggestionPopover
 {
   GtkWindow           parent_instance;
@@ -53,6 +57,8 @@ struct _DzlSuggestionPopover
 
   GListModel         *model;
 
+  GdkDevice          *grab_device;
+
   GType               row_type;
 
   gulong              delete_event_handler;
@@ -61,10 +67,15 @@ struct _DzlSuggestionPopover
   gulong              items_changed_handler;
   gulong              destroy_handler;
 
+  guint               queued_popdown;
+
   PangoEllipsizeMode  subtitle_ellipsize;
   PangoEllipsizeMode  title_ellipsize;
 
   guint               popup_requested : 1;
+  guint               entry_focused : 1;
+  guint               has_grab : 1;
+  guint               compact : 1;
 };
 
 enum {
@@ -490,6 +501,96 @@ attach_cb (DzlListBox    *list_box,
   _dzl_suggestion_row_set_ellipsize (DZL_SUGGESTION_ROW (row),
                                      self->title_ellipsize,
                                      self->subtitle_ellipsize);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (row), self->compact ? GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
+}
+
+static gboolean
+dzl_suggestion_popover_key_press_event (GtkWidget   *widget,
+                                        GdkEventKey *event)
+{
+  DzlSuggestionPopover *self = (DzlSuggestionPopover *)widget;
+
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+  g_assert (event != NULL);
+
+  if (self->relative_to != NULL)
+    {
+      DzlShortcutController *controller = dzl_shortcut_controller_try_find (self->relative_to);
+      g_autoptr(DzlShortcutChord) chord = NULL;
+
+      /* NOTE: This only allows for shortcuts on the target entry, not any
+       *       global shortcut. That is similar to how GtkEntryCompletion works
+       *       and ensures that we don't have state taken during global dispatch.
+       */
+
+      if (controller != NULL &&
+          (chord = dzl_shortcut_chord_new_from_event (event)))
+        {
+          DzlShortcutMatch match;
+
+          match = _dzl_shortcut_controller_handle (controller,
+                                                   event,
+                                                   chord,
+                                                   DZL_SHORTCUT_PHASE_DISPATCH,
+                                                   self->relative_to);
+
+          if (match == DZL_SHORTCUT_MATCH_EQUAL)
+            return TRUE;
+        }
+
+      return gtk_widget_event (self->relative_to, (GdkEvent *)event);
+    }
+
+  return GTK_WIDGET_CLASS (dzl_suggestion_popover_parent_class)->key_press_event (widget, event);
+}
+
+static gboolean
+dzl_suggestion_popover_key_release_event (GtkWidget   *widget,
+                                          GdkEventKey *event)
+{
+  DzlSuggestionPopover *self = (DzlSuggestionPopover *)widget;
+
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+  g_assert (event != NULL);
+
+  if (self->relative_to != NULL)
+    return gtk_widget_event (self->relative_to, (GdkEvent *)event);
+
+  return GTK_WIDGET_CLASS (dzl_suggestion_popover_parent_class)->key_release_event (widget, event);
+}
+
+static gboolean
+dzl_suggestion_popover_grab_broken_event (GtkWidget          *widget,
+                                          GdkEventGrabBroken *event)
+{
+  DzlSuggestionPopover *self = (DzlSuggestionPopover *)widget;
+
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+  g_assert (event != NULL);
+
+  dzl_suggestion_popover_popdown (self);
+
+  return GTK_WIDGET_CLASS (dzl_suggestion_popover_parent_class)->grab_broken_event (widget, event);
+}
+
+static gboolean
+dzl_suggestion_popover_button_release_event (GtkWidget      *widget,
+                                             GdkEventButton *event)
+{
+  DzlSuggestionPopover *self = (DzlSuggestionPopover *)widget;
+  gboolean ret;
+
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+  g_assert (event != NULL);
+
+  ret = GTK_WIDGET_CLASS (dzl_suggestion_popover_parent_class)->button_release_event (widget, event);
+
+  dzl_suggestion_popover_popdown (self);
+
+  /* Force immediate hide */
+  gtk_widget_hide (GTK_WIDGET (self));
+
+  return ret;
 }
 
 static void
@@ -497,6 +598,9 @@ dzl_suggestion_popover_destroy (GtkWidget *widget)
 {
   DzlSuggestionPopover *self = (DzlSuggestionPopover *)widget;
 
+  dzl_clear_source (&self->queued_popdown);
+  g_clear_object (&self->grab_device);
+
   dzl_suggestion_popover_set_transient_for (self, NULL);
 
   if (self->scroll_anim != NULL)
@@ -596,6 +700,10 @@ dzl_suggestion_popover_class_init (DzlSuggestionPopoverClass *klass)
   widget_class->screen_changed = dzl_suggestion_popover_screen_changed;
   widget_class->realize = dzl_suggestion_popover_realize;
   widget_class->show = dzl_suggestion_popover_show;
+  widget_class->button_release_event = dzl_suggestion_popover_button_release_event;
+  widget_class->key_press_event = dzl_suggestion_popover_key_press_event;
+  widget_class->key_release_event = dzl_suggestion_popover_key_release_event;
+  widget_class->grab_broken_event = dzl_suggestion_popover_grab_broken_event;
 
   properties [PROP_MODEL] =
     g_param_spec_object ("model",
@@ -703,6 +811,7 @@ dzl_suggestion_popover_new (void)
 void
 dzl_suggestion_popover_popup (DzlSuggestionPopover *self)
 {
+  GdkScreen *screen = NULL;
   guint duration = 250;
   guint n_items;
 
@@ -714,6 +823,9 @@ dzl_suggestion_popover_popup (DzlSuggestionPopover *self)
       return;
     }
 
+  if (gtk_widget_get_mapped (GTK_WIDGET (self)))
+    return;
+
   if (self->relative_to != NULL)
     {
       GdkDisplay *display;
@@ -726,6 +838,9 @@ dzl_suggestion_popover_popup (DzlSuggestionPopover *self)
       display = gtk_widget_get_display (GTK_WIDGET (self->relative_to));
       window = gtk_widget_get_window (GTK_WIDGET (self->relative_to));
       monitor = gdk_display_get_monitor_at_window (display, window);
+      screen = gtk_widget_get_screen (GTK_WIDGET (self->relative_to));
+
+      gtk_window_set_screen (GTK_WINDOW (self), screen);
 
       gtk_widget_get_preferred_height (GTK_WIDGET (self), &min_height, &nat_height);
       gtk_widget_get_allocation (GTK_WIDGET (self), &alloc);
@@ -733,8 +848,19 @@ dzl_suggestion_popover_popup (DzlSuggestionPopover *self)
       duration = dzl_animation_calculate_duration (monitor, alloc.height, nat_height);
     }
 
+  gtk_widget_grab_focus (GTK_WIDGET (self));
   gtk_widget_show (GTK_WIDGET (self));
 
+  if (!self->has_grab && self->grab_device != NULL)
+    {
+      self->has_grab = TRUE;
+      gtk_grab_add (GTK_WIDGET (self));
+      gdk_seat_grab (gdk_device_get_seat (self->grab_device),
+                     gtk_widget_get_window (GTK_WIDGET (self)),
+                     GDK_SEAT_CAPABILITY_POINTER | GDK_SEAT_CAPABILITY_TOUCH,
+                     TRUE, NULL, NULL, NULL, NULL);
+    }
+
   gtk_revealer_set_transition_duration (self->revealer, duration);
   gtk_revealer_set_reveal_child (self->revealer, TRUE);
 }
@@ -752,7 +878,20 @@ dzl_suggestion_popover_popdown (DzlSuggestionPopover *self)
 
   self->popup_requested = FALSE;
 
-  if (!gtk_widget_get_realized (GTK_WIDGET (self)))
+  if (self->has_grab)
+    {
+      self->has_grab = FALSE;
+      gtk_grab_remove (GTK_WIDGET (self));
+
+      if (self->grab_device != NULL)
+        {
+          gdk_seat_ungrab (gdk_device_get_seat (self->grab_device));
+          g_clear_object (&self->grab_device);
+        }
+    }
+
+
+  if (!gtk_widget_get_mapped (GTK_WIDGET (self)))
     return;
 
   display = gtk_widget_get_display (GTK_WIDGET (self->relative_to));
@@ -767,6 +906,30 @@ dzl_suggestion_popover_popdown (DzlSuggestionPopover *self)
   gtk_revealer_set_reveal_child (self->revealer, FALSE);
 }
 
+static gboolean
+dzl_suggestion_popover_do_queued_popdown (gpointer data)
+{
+  DzlSuggestionPopover *self = data;
+
+  self->queued_popdown = 0;
+
+  if (self->model != NULL && g_list_model_get_n_items (self->model) == 0)
+    dzl_suggestion_popover_popdown (self);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+dzl_suggestion_popover_queue_popdown (DzlSuggestionPopover *self)
+{
+  g_assert (DZL_IS_SUGGESTION_POPOVER (self));
+
+  dzl_clear_source (&self->queued_popdown);
+  self->queued_popdown = gdk_threads_add_timeout (DELAYED_POPDOWN_MSEC,
+                                                  dzl_suggestion_popover_do_queued_popdown,
+                                                  self);
+}
+
 static void
 dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
                                       guint                 position,
@@ -784,10 +947,16 @@ dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
 
   if (g_list_model_get_n_items (model) == 0)
     {
-      dzl_suggestion_popover_popdown (self);
+      /* do not immediately dismiss, because this might be an
+       * intermittant event only to be followed by an addition
+       * of new items to the GListModel.
+       */
+      dzl_suggestion_popover_queue_popdown (self);
       DZL_EXIT;
     }
 
+  dzl_clear_source (&self->queued_popdown);
+
   if (self->popup_requested)
     {
       dzl_suggestion_popover_popup (self);
@@ -795,6 +964,9 @@ dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
       DZL_EXIT;
     }
 
+  if (gtk_widget_get_mapped (GTK_WIDGET (self)))
+    return;
+
   /*
    * If we are currently animating in the initial view of the popover,
    * then we might need to cancel that animation and rely on the elastic
@@ -814,10 +986,11 @@ dzl_suggestion_popover_items_changed (DzlSuggestionPopover *self,
                                          G_CALLBACK (dzl_suggestion_popover_notify_child_revealed),
                                          self);
     }
-  else
+  else if (self->entry_focused)
     {
       dzl_suggestion_popover_popup (self);
       self->popup_requested = FALSE;
+      DZL_EXIT;
     }
 
   DZL_EXIT;
@@ -949,11 +1122,19 @@ dzl_suggestion_popover_move_by (DzlSuggestionPopover *self,
 
   g_return_if_fail (DZL_IS_SUGGESTION_POPOVER (self));
 
-  if (NULL == (row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), 0)))
+  if (!(row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), 0)))
     return;
 
-  if (NULL == gtk_list_box_get_selected_row (GTK_LIST_BOX (self->list_box)))
+  if (!gtk_list_box_get_selected_row (GTK_LIST_BOX (self->list_box)))
     {
+      if (amount < 0)
+        {
+          guint n_items = g_list_model_get_n_items (self->model);
+
+          if (n_items > 0)
+            row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->list_box), n_items - 1);
+        }
+
       dzl_suggestion_popover_select_row (self, row);
       return;
     }
@@ -1135,3 +1316,62 @@ _dzl_suggestion_popover_set_click_mode (DzlSuggestionPopover *self,
 
   gtk_list_box_set_activate_on_single_click (GTK_LIST_BOX (self->list_box), single_click);
 }
+
+void
+_dzl_suggestion_popover_set_focused (DzlSuggestionPopover *self,
+                                     gboolean              entry_focused)
+{
+  g_return_if_fail (DZL_IS_SUGGESTION_POPOVER (self));
+
+  self->entry_focused = !!entry_focused;
+  if (!entry_focused)
+    self->popup_requested = FALSE;
+}
+
+void
+_dzl_suggestion_popover_set_device (DzlSuggestionPopover *self,
+                                    GdkDevice            *device)
+{
+  g_return_if_fail (DZL_IS_SUGGESTION_POPOVER (self));
+  g_return_if_fail (!device || GDK_IS_DEVICE (device));
+
+  if (device != self->grab_device)
+    {
+      if (self->has_grab && self->grab_device != NULL)
+        gdk_seat_ungrab (gdk_device_get_seat (self->grab_device));
+      g_set_object (&self->grab_device, device);
+    }
+}
+
+static void
+make_rows_vertical (GtkWidget *row,
+                    gpointer   user_data)
+{
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (row), GTK_ORIENTATION_VERTICAL);
+}
+
+static void
+make_rows_horizontal (GtkWidget *row,
+                      gpointer   user_data)
+{
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (row), GTK_ORIENTATION_HORIZONTAL);
+}
+
+void
+_dzl_suggestion_popover_set_compact (DzlSuggestionPopover *self,
+                                     gboolean              compact)
+{
+  g_return_if_fail (DZL_IS_SUGGESTION_POPOVER (self));
+
+  compact = !!compact;
+
+  if (compact != self->compact)
+    {
+      self->compact = compact;
+
+      if (compact)
+        gtk_container_foreach (GTK_CONTAINER (self->list_box), make_rows_vertical, NULL);
+      else
+        gtk_container_foreach (GTK_CONTAINER (self->list_box), make_rows_horizontal, NULL);
+    }
+}
