@@ -20,25 +20,35 @@
 
 #include "config.h"
 
-#include "dzl-suggestion-row.h"
+#include "suggestions/dzl-suggestion-row.h"
+#include "util/dzl-macros.h"
 
 typedef struct
 {
   DzlSuggestion *suggestion;
 
+  GtkOrientation orientation;
+
+  gulong         notify_icon_handler;
+
   GtkImage      *image;
   GtkLabel      *title;
   GtkLabel      *separator;
   GtkLabel      *subtitle;
+  GtkLabel      *box;
 } DzlSuggestionRowPrivate;
 
 enum {
   PROP_0,
   PROP_SUGGESTION,
+  PROP_ORIENTATION,
   N_PROPS
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (DzlSuggestionRow, dzl_suggestion_row, DZL_TYPE_LIST_BOX_ROW)
+G_DEFINE_TYPE_EXTENDED (DzlSuggestionRow, dzl_suggestion_row, DZL_TYPE_LIST_BOX_ROW, 0,
+                        G_ADD_PRIVATE (DzlSuggestionRow)
+                        G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL))
+
 
 static GParamSpec *properties [N_PROPS];
 
@@ -52,48 +62,80 @@ dzl_suggestion_row_disconnect (DzlSuggestionRow *self)
   if (priv->suggestion == NULL)
     return;
 
+  dzl_clear_signal_handler (priv->suggestion, &priv->notify_icon_handler);
+
   g_object_set (priv->image, "icon-name", NULL, NULL);
   gtk_label_set_label (priv->title, NULL);
   gtk_label_set_label (priv->subtitle, NULL);
 }
 
 static void
-dzl_suggestion_row_connect (DzlSuggestionRow *self)
+on_notify_icon_cb (DzlSuggestionRow *self,
+                   GParamSpec       *pspec,
+                   DzlSuggestion    *suggestion)
 {
   DzlSuggestionRowPrivate *priv = dzl_suggestion_row_get_instance_private (self);
   cairo_surface_t *surface;
-  const gchar *subtitle;
 
-  g_return_if_fail (DZL_IS_SUGGESTION_ROW (self));
-  g_return_if_fail (priv->suggestion != NULL);
+  g_assert (DZL_IS_SUGGESTION_ROW (self));
+  g_assert (DZL_IS_SUGGESTION (suggestion));
 
-  if ((surface = dzl_suggestion_get_icon_surface (priv->suggestion, GTK_WIDGET (priv->image))))
+  if ((surface = dzl_suggestion_get_icon_surface (suggestion, GTK_WIDGET (priv->image))))
     {
       gtk_image_set_from_surface (priv->image, surface);
       cairo_surface_destroy (surface);
     }
   else
     {
-      g_autoptr(GIcon) icon = dzl_suggestion_get_icon (priv->suggestion);
+      g_autoptr(GIcon) icon = dzl_suggestion_get_icon (suggestion);
       gtk_image_set_from_gicon (priv->image, icon, GTK_ICON_SIZE_MENU);
     }
+}
+
+static void
+dzl_suggestion_row_connect (DzlSuggestionRow *self)
+{
+  DzlSuggestionRowPrivate *priv = dzl_suggestion_row_get_instance_private (self);
+  const gchar *subtitle;
+
+  g_return_if_fail (DZL_IS_SUGGESTION_ROW (self));
+  g_return_if_fail (priv->suggestion != NULL);
+
+  priv->notify_icon_handler =
+    g_signal_connect_object (priv->suggestion,
+                             "notify::icon",
+                             G_CALLBACK (on_notify_icon_cb),
+                             self,
+                             G_CONNECT_SWAPPED);
+
+  on_notify_icon_cb (self, NULL, priv->suggestion);
 
   gtk_label_set_label (priv->title, dzl_suggestion_get_title (priv->suggestion));
 
   subtitle = dzl_suggestion_get_subtitle (priv->suggestion);
   gtk_label_set_label (priv->subtitle, subtitle);
-  gtk_widget_set_visible (GTK_WIDGET (priv->separator), !!subtitle);
+
+  if (priv->orientation == GTK_ORIENTATION_VERTICAL)
+    gtk_widget_set_visible (GTK_WIDGET (priv->separator), FALSE);
+  else
+    gtk_widget_set_visible (GTK_WIDGET (priv->separator), !!subtitle);
+
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (priv->box), priv->orientation);
 }
 
 static void
-dzl_suggestion_row_finalize (GObject *object)
+dzl_suggestion_row_dispose (GObject *object)
 {
   DzlSuggestionRow *self = (DzlSuggestionRow *)object;
   DzlSuggestionRowPrivate *priv = dzl_suggestion_row_get_instance_private (self);
 
-  g_clear_object (&priv->suggestion);
+  if (priv->suggestion != NULL)
+    {
+      dzl_suggestion_row_disconnect (self);
+      g_clear_object (&priv->suggestion);
+    }
 
-  G_OBJECT_CLASS (dzl_suggestion_row_parent_class)->finalize (object);
+  G_OBJECT_CLASS (dzl_suggestion_row_parent_class)->dispose (object);
 }
 
 static void
@@ -103,6 +145,7 @@ dzl_suggestion_row_get_property (GObject    *object,
                                  GParamSpec *pspec)
 {
   DzlSuggestionRow *self = DZL_SUGGESTION_ROW (object);
+  DzlSuggestionRowPrivate *priv = dzl_suggestion_row_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -110,6 +153,10 @@ dzl_suggestion_row_get_property (GObject    *object,
       g_value_set_object (value, dzl_suggestion_row_get_suggestion (self));
       break;
 
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, priv->orientation);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -122,6 +169,7 @@ dzl_suggestion_row_set_property (GObject      *object,
                                  GParamSpec   *pspec)
 {
   DzlSuggestionRow *self = DZL_SUGGESTION_ROW (object);
+  DzlSuggestionRowPrivate *priv = dzl_suggestion_row_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -129,6 +177,16 @@ dzl_suggestion_row_set_property (GObject      *object,
       dzl_suggestion_row_set_suggestion (self, g_value_get_object (value));
       break;
 
+    case PROP_ORIENTATION:
+      if (priv->orientation != g_value_get_enum (value))
+        {
+          priv->orientation = g_value_get_enum (value);
+          gtk_widget_set_visible (GTK_WIDGET (priv->separator),
+                                  priv->orientation != GTK_ORIENTATION_VERTICAL);
+          gtk_orientable_set_orientation (GTK_ORIENTABLE (priv->box), priv->orientation);
+        }
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -140,7 +198,7 @@ dzl_suggestion_row_class_init (DzlSuggestionRowClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  object_class->finalize = dzl_suggestion_row_finalize;
+  object_class->dispose = dzl_suggestion_row_dispose;
   object_class->get_property = dzl_suggestion_row_get_property;
   object_class->set_property = dzl_suggestion_row_set_property;
 
@@ -151,13 +209,22 @@ dzl_suggestion_row_class_init (DzlSuggestionRowClass *klass)
                          DZL_TYPE_SUGGESTION,
                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_properties (object_class, N_PROPS, properties);
+  properties [PROP_ORIENTATION] =
+    g_param_spec_enum ("orientation",
+                       "Orientation",
+                       "Orientation",
+                       GTK_TYPE_ORIENTATION,
+                       GTK_ORIENTATION_VERTICAL,
+                       (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+   g_object_class_install_properties (object_class, N_PROPS, properties);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/dazzle/ui/dzl-suggestion-row.ui");
   gtk_widget_class_bind_template_child_private (widget_class, DzlSuggestionRow, image);
   gtk_widget_class_bind_template_child_private (widget_class, DzlSuggestionRow, title);
   gtk_widget_class_bind_template_child_private (widget_class, DzlSuggestionRow, subtitle);
   gtk_widget_class_bind_template_child_private (widget_class, DzlSuggestionRow, separator);
+  gtk_widget_class_bind_template_child_private (widget_class, DzlSuggestionRow, box);
 }
 
 static void
